# @push.rocks/smartlog-destination-local
a smartlog destination targeting the local console

## Install
To use `@push.rocks/smartlog-destination-local` in your project, you need to install it via npm. You can do this by running:

```bash
npm install @push.rocks/smartlog-destination-local --save
```

This command adds `@push.rocks/smartlog-destination-local` to your project's dependencies and enables you to start implementing its features in your application.

## Usage

The `@push.rocks/smartlog-destination-local` package provides a way to easily log messages to the local console with support for color coding and structured message formats. It is especially useful when combined with the `@push.rocks/smartlog` package to create a powerful logging solution that is both visually informative and structured for better logging practices.

### Importing and Setting Up

First, make sure to import the package where you intend to use it:

```typescript
import { DestinationLocal } from '@push.rocks/smartlog-destination-local';
```

Next, create an instance of `DestinationLocal`:

```typescript
const localLogDestination = new DestinationLocal();
```

### Logging Messages

The primary feature of `@push.rocks/smartlog-destination-local` is to log messages to the console. This can be done by preparing a log package and using the `handleLog` method.

#### Example Log Message

```typescript
import type { ILogPackage } from '@push.rocks/smartlog-interfaces';

const logPackage: ILogPackage = {
  timestamp: Date.now(),
  type: 'log', // or 'error', 'warn', etc. depending on the nature of the message
  level: 'info', // determines the visibility and importance of the log
  context: {
    company: 'Your Company',
    companyunit: 'Your Unit',
    containerName: 'Your Container',
    environment: 'development',
    runtime: 'node',
    zone: 'Your Zone'
  },
  message: 'This is a test log message.', // the actual message
  correlation: {
    id: 'uniqueId', // useful for tracing logs
    type: 'requestId'
  },
};

// Using the instance to log the message
localLogDestination.handleLog(logPackage);
```

This will output a structured, color-coded log message to your console, making it easier to differentiate between various log levels and to understand the context of the log messages at a glance.

### Log Reduction

For scenarios where repeated log messages may clutter your log output, `@push.rocks/smartlog-destination-local` offers a `logReduced` method. This method will throttle the output of identical consecutive messages according to the settings you provide.

#### Reducing Log Output

```typescript
// Log a message repeatedly, e.g., in a loop
for (let i = 0; i < 100; i++) {
  localLogDestination.logReduced('Repeated message', 5); // only logs every 5th identical message
}
```

### Adding New Lines

You may want to improve the readability of your logs by inserting blank lines. This can be easily achieved:

```typescript
localLogDestination.newLine(2); // Inserts two empty lines in the log output
```

### Advanced Use Cases

The `@push.rocks/smartlog-destination-local` package is flexible and integrates seamlessly with other logging and monitoring solutions. It can be extended or customized to fit more complex logging requirements, such as filtering logs based on environment, enhancing log messages with additional metadata, or integrating with remote logging services for centralized log management.

### Conclusion

`@push.rocks/smartlog-destination-local` serves as a robust, easy-to-use solution for enhancing your application's logging capabilities. By following the examples and using the provided methods, you can quickly implement effective logging practices that improve debugging, monitoring, and analysis of your application's behavior in development and production environments.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
